@@ -318,15 +318,14 @@ BUILTIN_THEMES: dict[str, Theme] = {
         panel="#26233a",
         dark=True,
         variables={
-            "input-cursor-foreground": "#524f67",
-            "input-cursor-background": "#21202e",
+            "input-cursor-background": "#f4ede8",
             "input-selection-background": "#403d52",
             "border": "#524f67",
             "border-blurred": "#6e6a86",
             "footer-background": "#26233a",
-            "block-cursor-foreground": "#524f67",
+            "block-cursor-foreground": "#191724",
             "block-cursor-text-style": "none",
-            "button-color-foreground": "#403d52",
+            "block-cursor-background": "#c4a7e7",
         },
     ),
     "rose-pine-moon": Theme(
@@ -343,15 +342,14 @@ BUILTIN_THEMES: dict[str, Theme] = {
         panel="#393552",
         dark=True,
         variables={
-            "input-cursor-foreground": "#56526e",
-            "input-cursor-background": "#2a283e",
+            "input-cursor-background": "#f4ede8",
             "input-selection-background": "#44415a",
             "border": "#56526e",
             "border-blurred": "#6e6a86",
             "footer-background": "#393552",
-            "block-cursor-foreground": "#56526e",
+            "block-cursor-foreground": "#232136",
             "block-cursor-text-style": "none",
-            "button-color-foreground": "#44415a",
+            "block-cursor-background": "#c4a7e7",
         },
     ),
     "rose-pine-dawn": Theme(
@@ -368,15 +366,14 @@ BUILTIN_THEMES: dict[str, Theme] = {
         panel="#f2e9e1",
         dark=False,
         variables={
-            "input-cursor-foreground": "#cecacd",
-            "input-cursor-background": "#f4ede8",
+            "input-cursor-background": "#575279",
             "input-selection-background": "#dfdad9",
             "border": "#cecacd",
             "border-blurred": "#9893a5",
             "footer-background": "#f2e9e1",
-            "block-cursor-foreground": "#cecacd",
+            "block-cursor-foreground": "#faf4ed",
             "block-cursor-text-style": "none",
-            "button-color-foreground": "#dfdad9",
+            "block-cursor-background": "#575279",
         },
     ),
 }
